#ifndef MODULE_BITMAP_BITMAP_COMPOSITE_SIMPLE_H
#define MODULE_BITMAP_BITMAP_COMPOSITE_SIMPLE_H

// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\author Anders Dahnielson (anders@dahnielson.com)
*/

#include <k3dsdk/bitmap.h>
#include <k3dsdk/node.h>
#include <k3dsdk/persistent.h>

#include "bitmap_composite.h"
#include "bitmap_element.h"

namespace libk3dbitmap
{

class bitmap_composite_simple :
	public bitmap_composite<k3d::persistent<k3d::node> >
{
	typedef bitmap_composite<k3d::persistent<k3d::node> > base;

public:
	bitmap_composite_simple(k3d::idocument& Document) :
		base(Document)
	{
		m_bitmap_a.changed_signal().connect(sigc::mem_fun(*this, &bitmap_composite_simple::on_reset_bitmap));
		m_bitmap_b.changed_signal().connect(sigc::mem_fun(*this, &bitmap_composite_simple::on_reset_bitmap));
	}

	void on_reset_bitmap(k3d::iunknown*)
	{
		m_bitmap_o.reset();
	}

	void on_create_bitmap(k3d::bitmap& Output)
	{
		// If we don't have any input bitmap, we're done ...
		k3d::bitmap* const bitmap_a = m_bitmap_a.value();
		k3d::bitmap* const bitmap_b = m_bitmap_b.value();
		if(!bitmap_a && !bitmap_b)
		{
			return;
		}
		else if(!bitmap_b)
		{
			Output = *bitmap_a;
			return;
		}
		else if(!bitmap_a)
		{
			Output = *bitmap_b;
			return;
		}

		// Update values
		on_value_change(0);

		// Composite ...
		bitmap_element element_a(*bitmap_a);
		bitmap_element element_b(*bitmap_b);

		const int max_x = std::max(element_a.max_x, element_b.max_x);
		const int min_x = std::min(element_a.min_x, element_b.min_x);
		const int max_y = std::max(element_a.max_y, element_b.max_y);
		const int min_y = std::min(element_a.min_y, element_b.min_y);

		Output.reset(max_x - min_x, max_y - min_y);
		k3d::bitmap::iterator pixel_output = Output.begin();

		for(int y = max_y; y > min_y; --y)
		{
			for(int x = min_x; x < max_x; ++x)
			{
				k3d::pixel pixel_a = element_a.get_pixel(x, y);
				k3d::pixel pixel_b = element_b.get_pixel(x, y);
				on_composite(pixel_a, pixel_b, *pixel_output);
				++pixel_output;
			}
		}
	}

	virtual void on_value_change(k3d::iunknown*) = 0;
	virtual void on_composite(const k3d::pixel& A, const k3d::pixel& B, k3d::pixel& O) = 0;
};

} // namespace libk3dbitmap

#endif // !MODULE_BITMAP_BITMAP_COMPOSITE_SIMPLE_H

