// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
	\author Anders Dahnielson (anders@dahnielson.com)
*/

#include <k3dsdk/i18n.h>
#include <k3dsdk/auto_ptr.h>
#include <k3dsdk/bitmap_source.h>
#include <k3dsdk/file_filter.h>
#include <k3dsdk/ibitmap_read_format.h>
#include <k3dsdk/itime_sink.h>
#include <k3dsdk/measurement.h>
#include <k3dsdk/module.h>
#include <k3dsdk/node.h>
#include <k3dsdk/options.h>
#include <k3dsdk/persistent.h>
#include <k3dsdk/property.h>
#include <k3dsdk/time_source.h>

#include <k3dsdk/fstream.h>

#include <fstream>
#include <iomanip>
#include <iterator>

namespace libk3dbitmap
{

/////////////////////////////////////////////////////////////////////////////
// bitmap_sequence_in

class bitmap_sequence_in :
	public k3d::bitmap_source<k3d::persistent<k3d::node> >,
	public k3d::itime_sink
{
	typedef k3d::bitmap_source<k3d::persistent<k3d::node> > base;

public:
	bitmap_sequence_in(k3d::idocument& Document) :
		base(Document),
		m_file(init_owner(*this) + init_name("file_path") + init_label(_("File path")) + init_description(_("Path for bitmap sequence")) + init_value(k3d::filesystem::path()) + init_path_mode(k3d::ipath_property::READ) + init_path_type(k3d::options::path::bitmaps())),
		m_time(init_owner(*this) + init_name("time") + init_label(_("Input time")) + init_description(_("Bitmap change delay")) + init_value(0.0))
	{
		m_file.changed_signal().connect(make_reset_bitmap_slot());
		m_time.changed_signal().connect(make_reset_bitmap_slot());
	}

	k3d::iproperty& time_sink_input()
	{
		return m_time;
	}

	void on_create_bitmap(k3d::bitmap& Bitmap)
	{
		const k3d::filesystem::path file = m_file.value();

		// Check that the document has animation capabilities ...
		if(k3d::iproperty* const frame_rate_property = k3d::get_frame_rate(document()))
		{
assert_not_implemented();
/*
			// Solve frame number and filename ...
			const double frame_rate = boost::any_cast<double>(k3d::get_value(document().dag(), *frame_rate_property));
			const long view_frame = static_cast<long>(k3d::round(frame_rate * m_time.value()));

			k3d::frames frames(file);
			k3d::filesystem::path destination;
			frames.frame(view_frame, destination);

			// Read file ...
			if(!k3d::filesystem::exists(destination))
				return;

			k3d::auto_ptr<k3d::ibitmap_read_format> filter(k3d::auto_file_filter<k3d::ibitmap_read_format>(destination));

			return_if_fail(filter.get());
			filter->read_file(destination, Bitmap);
*/
		}
		else
		{
			// Read file ...
			if (!k3d::filesystem::exists(file))
				return;

			k3d::auto_ptr<k3d::ibitmap_read_format> filter(k3d::auto_file_filter<k3d::ibitmap_read_format>(file));

			return_if_fail(filter.get());
			filter->read_file(file, Bitmap);
		}
	}

	void on_update_bitmap(k3d::bitmap& Bitmap)
	{
	}

	k3d::iplugin_factory& factory()
	{
		return get_factory();
	}

	static k3d::iplugin_factory& get_factory()
	{
		static k3d::plugin_factory<
			k3d::document_plugin<bitmap_sequence_in>,
			k3d::interface_list<k3d::ibitmap_source,
			k3d::interface_list<k3d::itime_sink> > > factory(
				k3d::uuid(0x23ec2e00, 0x53064bed, 0xa5bf23d9, 0x464a8db4),
				"BitmapSequenceIn",
				_("Load sequence of bitmaps from the filesystem"),
				"Bitmap",
				k3d::iplugin_factory::STABLE);

		return factory;
	}

private:
	k3d_data(k3d::filesystem::path, immutable_name, change_signal, with_undo, local_storage, no_constraint, path_property, path_serialization) m_file;
	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_time;
};

/////////////////////////////////////////////////////////////////////////////
// bitmap_sequence_in_factory

k3d::iplugin_factory& bitmap_sequence_in_factory()
{
	return bitmap_sequence_in::get_factory();
}

} // namespace libk3dbitmap

