// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\author Anders Dahnielson (anders@dahnielson.com)
*/

#include "simple_bitmap_modifier.h"

#include <k3dsdk/i18n.h>
#include <k3dsdk/module.h>

namespace libk3dbitmap
{

/////////////////////////////////////////////////////////////////////////////
// bitmap_sub

class bitmap_sub :
	public simple_bitmap_modifier
{
	typedef simple_bitmap_modifier base;

public:
	bitmap_sub(k3d::idocument& Document) :
		base(Document),
		m_value(init_owner(*this) + init_name("value") + init_label(_("Subtract value")) + init_description(_("Subtract value to each pixel color component")) + init_value(0.0))
	{
		m_value.changed_signal().connect(make_update_bitmap_slot());
	}

	void on_update_bitmap(const k3d::bitmap& Input, k3d::bitmap& Output)
	{
		const double value = m_value.value();

		k3d::bitmap::const_iterator input = Input.begin();
		const k3d::bitmap::const_iterator end = Input.end();

		k3d::bitmap::iterator output = Output.begin();

		for(; input != end; ++input, ++output)
		{
			output->red = static_cast<double>(input->red) - value;
			output->green = static_cast<double>(input->green) - value;
			output->blue = static_cast<double>(input->blue) - value;
			output->alpha = input->alpha;
		}
	}

	k3d::iplugin_factory& factory()
	{
		return get_factory();
	}

	static k3d::iplugin_factory& get_factory()
	{
		static k3d::plugin_factory<
			k3d::document_plugin<bitmap_sub>,
			k3d::interface_list<k3d::ibitmap_source,
			k3d::interface_list<k3d::ibitmap_sink> > > factory(
				k3d::uuid(0x51c8f52f, 0x53834926, 0x865e3592, 0xf0d09510),
				"BitmapSubtract",
				_("Subtract value from each pixel"),
				"Bitmap",
				k3d::iplugin_factory::STABLE);

		return factory;
	}

private:
	k3d_data(double, immutable_name, change_signal, with_undo, local_storage, no_constraint, writable_property, with_serialization) m_value;
	double m_value_cache;
};

/////////////////////////////////////////////////////////////////////////////
// bitmap_sub_factory

k3d::iplugin_factory& bitmap_sub_factory()
{
	return bitmap_sub::get_factory();
}

} // namespace libk3dbitmap


