// K-3D
// Copyright (c) 1995-2005, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the RenderManScript K-3D object, which can insert scripted data into RenderMan output
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dsdk/i18n.h>
#include <k3dsdk/mesh_modifier.h>
#include <k3dsdk/module.h>
#include <k3dsdk/node.h>
#include <k3dsdk/persistent.h>
#include <k3dsdk/scripted_node.h>

#define DEFAULT_SCRIPT "#python\n\nimport k3d\n\nk3d.deep_copy(Input, Output)\n\n"

namespace libk3dscripting
{

/////////////////////////////////////////////////////////////////////////////
// mesh_modifier_script

class mesh_modifier_script :
	public k3d::scripted_node<k3d::mesh_modifier<k3d::persistent<k3d::node> > >
{
	typedef k3d::scripted_node<k3d::mesh_modifier<k3d::persistent<k3d::node> > > base;

public:
	mesh_modifier_script(k3d::idocument& Document) :
		base(Document)
	{
		set_script(DEFAULT_SCRIPT);

		connect_script_changed_signal(make_reset_mesh_slot());
	}

	/** \todo Improve the implementation so we don't have to do this */
	k3d::iunknown* on_rewrite_hint(iunknown* const Hint)
	{
		// Force updates to re-allocate our mesh, for simplicity
		return 0;
	}

	void on_create_mesh(const k3d::mesh& Input, k3d::mesh& Mesh)
	{
		k3d::iscript_engine::context_t context;
		context["Document"] = static_cast<k3d::iunknown*>(&document());
		context["Node"] = static_cast<k3d::iunknown*>(this);
		context["Input"] = static_cast<k3d::iunknown*>(const_cast<k3d::mesh*>(&Input));
		context["Output"] = static_cast<k3d::iunknown*>(&Mesh);

		execute_script(context);
	}

	void on_update_mesh(const k3d::mesh& Input, k3d::mesh& Mesh)
	{
	}

	k3d::iplugin_factory& factory()
	{
		return get_factory();
	}

	static k3d::iplugin_factory& get_factory()
	{
		static k3d::plugin_factory<
			k3d::document_plugin<mesh_modifier_script>,
			k3d::interface_list<k3d::imesh_sink, k3d::interface_list<k3d::imesh_source> > > factory(
			k3d::uuid(0xd51fe8e4, 0x5aad4790, 0x82957933, 0xb22c1c1c),
			"MeshModifierScript",
			_("Mesh modifier that uses a script to modify geometry"),
			"Scripting Mesh",
			k3d::iplugin_factory::STABLE);

		return factory;
	}
};

/////////////////////////////////////////////////////////////////////////////
// mesh_modifier_script_factory

k3d::iplugin_factory& mesh_modifier_script_factory()
{
	return mesh_modifier_script::get_factory();
}

} // namespace libk3dscripting

